@@ -3,14 +3,13 @@
 #include <log-functions.h>
 
 bool
-Write_Stream(Stream &file, 
+Write_Stream(Stream &, 
 	     const Server_Image &server_image)
 {
 	assert (server_image.Is());
 
 	if (! server_image.Has_Canvas())
 		return false;
-//	Canvas ca;
 
 	Client_Image client_image;
 	if (! client_image.Receive_Rect(server_image(), 
@@ -18,8 +17,7 @@ Write_Stream(Stream &file,
 				  server_image.Get_Size())))
 		return false;
 
-	return Write_Stream(file,
-			  client_image);
+    return true;
 }
 
 bool
