@@ -26,7 +26,6 @@
 #include <errno.h>
 #include <string.h>
 #include <sys/vfs.h>
-//#include "../chessui_i18n.hh"
 #include <libgen.h>
 #include <gtk/gtk.h>
 
@@ -76,11 +75,15 @@
 #define DEFAULT_STATE_SAVE_FILE            "/tmp/osso-chess-save"
 #define CHESS_PAUSE_SAVE_FILE_TMP "/tmp/chess_state_file.pgn"
 #define CHESS_SAVE_FILE_TMP "/tmp/osso_chess.tmp"
-// #define CHESS_OBEX_COPY_FILE_TMP "/tmp/osso_chess_obex_copy.game"
 #define CHESS_SERVICE "com.nokia.osso_chess"
 #define CHESS_OBJPATH "/com/nokia/osso_chess"
 #define CHESS_IFACE "com.nokia.osso_chess"
 
+/* When a (saved or new) game is loading, applicatin won't let the user start
+ * a new one with full-screen hw-key. This gconf key is set to TRUE when
+ * loading starts and set back to FALSE when finished. */
+#define SETTINGS_CHESS_LOAD_IN_PROGRESS "/apps/osso/chess/load_in_progress"
+
 #define MENU_RECENT_MENU_WIDTH 15
 
 #define GAME_SAVE_METHOD     "game_save"
@@ -98,7 +101,6 @@ gchar *mmc_uri;
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
-//#define _(String) gettext(String)
 #ifdef gettext_noop
 #define N_(String) gettext_noop(String)
 #else
@@ -129,7 +131,6 @@ gchar *mmc_uri;
 #define CHESS_MENU_OPPONENT_HUMAN      6
 #define CHESS_MENU_SOUND               7
 #define CHESS_MENU_SHOW_LEGAL_MOVES    8
-//#define CHESS_MENU_9     9
 
 /* menu_action identifiers */
 #define MA_GAME_PLAY      1
@@ -171,7 +172,6 @@ GtkWidget *human_box = NULL;
 GtkWidget *opponent_box = NULL;
 GtkWidget *sound_check = NULL;
 GtkWidget *show_legal_moves_check = NULL;
-//GnomeVFSMonitorHandle *gnomevfs_handle = NULL;
 gchar *chess_title  = NULL;
 osso_context_t *osso;
 
@@ -497,14 +497,9 @@ load_plugin (void)
   bindtextdomain (GETTEXT_PACKAGE, CHESSLOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   gcc = gconf_client_get_default();
-  //enable_sound = gconf_client_get_bool (gcc, SETTINGS_ENABLE_SOUND, NULL);
 	enable_sound = plugin_settings_get_bool_fallback(SETTINGS_ENABLE_SOUND,TRUE);
-  //show_legal_moves = gconf_client_get_bool (gcc, SETTINGS_SHOW_LEGAL_MOVES, NULL);
 	show_legal_moves =plugin_settings_get_bool_fallback(SETTINGS_SHOW_LEGAL_MOVES,FALSE);
-  //color_white = !gconf_client_get_bool (gcc, SETTINGS_PLAYER_COLOR, NULL);
 	color_white = !plugin_settings_get_bool_fallback(SETTINGS_PLAYER_COLOR,TRUE);
-  //opponent_type = gconf_client_get_bool (gcc, SETTINGS_OPPONENT_HUMAN, NULL);
-  //difficulty_level = gconf_client_get_int (gcc, SETTINGS_LEVEL, NULL);
   difficulty_level = plugin_settings_get_int_fallback(SETTINGS_LEVEL,0);
 
   game_vbox = gtk_vbox_new (FALSE, 0);
@@ -513,8 +508,6 @@ load_plugin (void)
   game_hbox = gtk_hbox_new (TRUE, 0);
   g_assert (game_hbox);
 
-  //hbox = gtk_hbox_new(FALSE,0);
-  //g_assert(hbox);
   
   
 
@@ -530,34 +523,17 @@ load_plugin (void)
   gtk_combo_box_set_active (GTK_COMBO_BOX (human_box), color_white);
   human_label =hildon_caption_new(groupleft,_("game_fi_chess_settings_human"),human_box,NULL,HILDON_CAPTION_OPTIONAL);
 	g_assert(human_label);
-  //hildon_caption_set_separator(HILDON_CAPTION(human_label)," ");
-  
 
   gtk_box_pack_start (GTK_BOX (game_hbox), game_vbox, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (game_vbox), human_label, FALSE, FALSE, 2);
 
-  //hbox = gtk_hbox_new(FALSE, 0);
-  
   sound_check = gtk_check_button_new ();
   g_assert (sound_check);
   sound_label = hildon_caption_new(groupright,_("game_fi_chess_settings_sound"),sound_check,NULL,HILDON_CAPTION_OPTIONAL);
 	g_assert (sound_label);
-  //hildon_caption_set_separator(HILDON_CAPTION(sound_label)," ");
-  
-  //gtk_box_pack_start (GTK_BOX (game_hbox),  sound_label, FALSE, FALSE, 2);
-
-  
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(sound_check), enable_sound);
 
-  //gtk_box_pack_start (GTK_BOX (game_vbox), game_hbox, FALSE, FALSE, 2);
-
-  //game_hbox = gtk_hbox_new (FALSE, 0);
-  //g_assert (game_hbox);
-  //hbox = gtk_hbox_new (FALSE, 0);
-  //g_assert(hbox);
-  
-  
   opponent_box = gtk_combo_box_new_text ();
   g_assert (opponent_box);
   gtk_combo_box_set_row_separator_func(GTK_COMBO_BOX(opponent_box),plugin_select_separator,NULL,NULL);
@@ -576,22 +552,16 @@ load_plugin (void)
     _("game_va_chess_settings_opponent_human")); //  "Human player");
   gtk_widget_set_size_request(opponent_box,
     COMBOBOX_WIDTH, COMBOBOX_HEIGHT);
-  /*if (opponent_type) {
-    gtk_combo_box_set_active (GTK_COMBO_BOX (opponent_box), 4);
-  } else {*/
   if (difficulty_level==4)
   {
     difficulty_level=5;
   }
     gtk_combo_box_set_active (GTK_COMBO_BOX (opponent_box), difficulty_level);
-  //}
 	opponent_label =hildon_caption_new(groupleft,_("game_fi_chess_settings_opponent"),opponent_box,NULL,HILDON_CAPTION_OPTIONAL);
 	g_assert(opponent_label);
-	//hildon_caption_set_separator(HILDON_CAPTION(opponent_label)," ");
 
 	gtk_box_pack_start (GTK_BOX (game_vbox),   opponent_label, FALSE, FALSE, 2);
 
-  //hbox = gtk_hbox_new(FALSE, 0);
   game_vbox = gtk_vbox_new (FALSE, 0);
 	g_assert(game_vbox);
 	
@@ -604,14 +574,11 @@ load_plugin (void)
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_legal_moves_check), show_legal_moves);
 	show_legal_moves_label = hildon_caption_new(groupright,_("game_fi_chess_settings_legal_moves"),show_legal_moves_check,NULL,HILDON_CAPTION_OPTIONAL);
 	g_assert (show_legal_moves_label);
-	//hildon_caption_set_separator(HILDON_CAPTION(show_legal_moves_label)," ");
   gtk_widget_set_size_request(show_legal_moves_label,
     LABEL_WIDTH, LABEL_HEIGHT);
 		
   gtk_box_pack_start (GTK_BOX (game_vbox), show_legal_moves_label, FALSE, FALSE, 2);
-  //gtk_box_pack_start (GTK_BOX (game_vbox), game_hbox, FALSE, FALSE, 2);
 
-  //gtk_widget_show_all (game_vbox);
 	gtk_widget_show_all (game_hbox);
   g_signal_connect (G_OBJECT(human_box), "changed",
       G_CALLBACK(chess_human_cb), NULL);
@@ -627,6 +594,7 @@ load_plugin (void)
   g_signal_connect (G_OBJECT(show_legal_moves_check), "clicked",
       G_CALLBACK(chess_show_cb), NULL);
   plugin_settings_set_string(SETTINGS_CHESS_SAVE_FILE, "");
+  plugin_settings_set_bool(SETTINGS_CHESS_LOAD_IN_PROGRESS, FALSE);
 	return game_hbox;  
 }
 
@@ -634,6 +602,8 @@ static void
 unload_plugin (void)
 {
 
+	plugin_restore_last_settings();
+
 	if( monitor_handle ) {
 		gnome_vfs_monitor_cancel( monitor_handle );
 		monitor_handle = NULL;
@@ -662,17 +632,11 @@ write_config (void)
 	gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (sound_check)), NULL);
     gconf_client_set_bool (gcc, SETTINGS_SHOW_LEGAL_MOVES, 
 	gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (show_legal_moves_check)), NULL);
-  //if (combo == 4) {
-    //gconf_client_set_int (gcc, SETTINGS_LEVEL, 0, NULL);
-    //gconf_client_set_bool (gcc, SETTINGS_OPPONENT_HUMAN, 1, NULL);
-    //gconf_client_set_bool (gcc, SETTINGS_PLAYER_COLOR, 0, NULL);
-  //} else {
   if (combo==5)
   {
     combo=4;
   }
     gconf_client_set_int (gcc, SETTINGS_LEVEL, combo, NULL);
-    //gconf_client_set_bool (gcc, SETTINGS_OPPONENT_HUMAN, 0, NULL);
     gconf_client_set_bool (gcc, SETTINGS_PLAYER_COLOR, 
     	!gtk_combo_box_get_active (GTK_COMBO_BOX (human_box)), NULL);
 	if (combo == 4) {
@@ -682,7 +646,6 @@ write_config (void)
 	{
 		gconf_client_set_bool (gcc, SETTINGS_OPPONENT_HUMAN, 0, NULL);
 	}
-  //}
 }
 
 static void game_help_cb (GtkMenuItem *menuitem, gpointer user_data)
@@ -856,10 +819,6 @@ plugin_callback (GtkWidget *menu_item,
     case CHESS_MENU_HUMAN_BLACK:
       if (!changed) {
         changed = TRUE;
-/*        if (gtk_combo_box_get_active (GTK_COMBO_BOX (opponent_box)) == CHESS_MENU_OPPONENT_HUMAN-2){
-          gtk_combo_box_set_active (GTK_COMBO_BOX (human_box), CHESS_MENU_HUMAN_WHITE);
-          gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(human_white_item), TRUE);  
-        } else {*/
         if ((int) data==4)
         {
           gtk_combo_box_set_active (GTK_COMBO_BOX (human_box), (int) 5);
@@ -868,7 +827,6 @@ plugin_callback (GtkWidget *menu_item,
         { 
           gtk_combo_box_set_active (GTK_COMBO_BOX (human_box), (int) data);
         }
-        //}
       }
 	  break;
     case CHESS_MENU_OPPONENT_EASY:
@@ -878,10 +836,6 @@ plugin_callback (GtkWidget *menu_item,
       if (!changed) {
         changed = TRUE;
         gtk_combo_box_set_active (GTK_COMBO_BOX (opponent_box), (int) data - 2);
-        /*if ((int) data == CHESS_MENU_OPPONENT_HUMAN){
-          gtk_combo_box_set_active (GTK_COMBO_BOX (human_box), CHESS_MENU_HUMAN_WHITE);
-          gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(human_white_item), TRUE); 
-		}*/
       }
       break;
     case CHESS_MENU_OPPONENT_HUMAN:
@@ -924,7 +878,6 @@ chess_human_cb (GtkWidget *widget,
       changed = TRUE;
       gint active = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
       if (active == CHESS_MENU_HUMAN_BLACK)
-                 //&& (gtk_combo_box_get_active (GTK_COMBO_BOX (opponent_box)) != CHESS_MENU_OPPONENT_HUMAN-2)) {
 	  {
         gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(human_black_item), TRUE);
       } else {
@@ -966,8 +919,6 @@ chess_opponent_cb (GtkWidget *widget,
         break;
       case 5:
         gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(opponent_human_item), TRUE);
-		//gtk_combo_box_set_active (GTK_COMBO_BOX (human_box), CHESS_MENU_HUMAN_WHITE);
-        //gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(human_white_item), TRUE); 
         break;
     }
   }
@@ -1043,8 +994,6 @@ static void game_save_cb(void)
 
   gnome_vfs_file_info_unref(file_info);
 
-  //ui_update_status((AppUIData*)data);
-
   if (file != NULL) 
     g_free(file);
 }
@@ -1079,7 +1028,6 @@ static void game_save_as_cb(void)
     game_save();
         
   }
-  //ui_update_status((AppUIData*)data);
   
   if (old_filename) {
     g_free(old_filename);
@@ -1235,12 +1183,6 @@ ui_show_file_chooser(gboolean open)
             chess_save_folder = pathname;
         }
         
-        /* Validate file if open */
-        if( open && !validate_chess_file( filename ) ) {
-        	g_free( filename );
-        	return FALSE;
-        }
-        
 		if( monitor_handle ) {
 			gnome_vfs_monitor_cancel( monitor_handle );
 			monitor_handle = NULL;
@@ -1274,11 +1216,25 @@ ui_show_file_chooser(gboolean open)
 
 
 
+static gboolean
+key_press_for_cancelling_dialog(GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+    if(event == NULL)
+        return FALSE;
+    if (event->keyval == GDK_Escape) {
+        gtk_dialog_response(GTK_DIALOG(widget), GTK_RESPONSE_CANCEL);
+        return TRUE;
+    }
+
+    data = 0;
+    return FALSE;
+}
+
 /* Show the replace dialog */
 static gboolean 
 ui_show_replace_file_dialog(gchar *file)
 {
-  gboolean answer = FALSE;
+  gint answer;
   gchar *fname=NULL,*bname=NULL;
   bname=g_path_get_basename(file);
   fname=gnome_vfs_unescape_string(bname,NULL);
@@ -1293,11 +1249,12 @@ ui_show_replace_file_dialog(gchar *file)
         D_("docm_bd_replace_file_cancel"),
         0,
         NULL));
+  g_signal_connect(G_OBJECT(note), "key-press-event", G_CALLBACK(key_press_for_cancelling_dialog), NULL);
   answer = gtk_dialog_run(GTK_DIALOG(note));
   gtk_widget_destroy(GTK_WIDGET(note));
 
   g_free(tmp);
-  return answer;
+  return (GTK_RESPONSE_OK == answer);
 }
 
 static void set_all_insensitive(void)
@@ -1324,6 +1281,10 @@ static void set_all_sensitive(void)
 	gtk_widget_set_sensitive(opponent_label,TRUE);
 	gtk_widget_set_sensitive(sound_label,TRUE);
 	gtk_widget_set_sensitive(show_legal_moves_label,TRUE);
+    if (GTK_WIDGET_VISIBLE(gs.ui->play_button))
+	{
+		gtk_widget_set_sensitive(gs.ui->play_button,TRUE);
+	}
 }
 
 /* The following function is from INDT's games-startup code. The copyrights 
@@ -1381,8 +1342,6 @@ plugin_restore_last_settings(void)
         selected_difficulty = -1;
     }
 
-    /**/fprintf(stderr, "restore last settings -- selected_human: %d\n",
-            selected_player);
     if ((selected_player != chess_human_invalid))
     {
         gtk_combo_box_set_active(GTK_COMBO_BOX(human_box),
@@ -1417,8 +1376,6 @@ static void plugin_restore_original_settins(void)
     gboolean player_current_value =
         ((gtk_combo_box_get_active(GTK_COMBO_BOX(human_box)) == 
             chess_human_white) ? TRUE : FALSE);
-    /**/fprintf(stderr, "restore original settings -- orig: %d, current: %d\n",
-            player_original_settings, player_current_value);
     if (player_original_settings != player_current_value)
     {
         selected_player =
@@ -1438,7 +1395,6 @@ static void plugin_restore_original_settins(void)
 static void 
 plugin_cb(GtkWidget *menu_item, gpointer cb_data)
 {
-    osso_log(LOG_DEBUG,"Plugin cb: %d",(int) cb_data );
   switch( (int) cb_data ) {
     case ME_GAME_OPEN:
       game_load_cb();
@@ -1494,6 +1450,7 @@ static void plugin_show_infoprint(gchar *msg)
 /* Show load error dialog */
 static void plugin_ui_show_load_err(int err_type, gchar *loadfile)
 { 
+  plugin_settings_set_bool(SETTINGS_CHESS_LOAD_IN_PROGRESS, FALSE);
   plugin_remove_recent_item(loadfile);
 
   switch (err_type) {
@@ -1505,7 +1462,9 @@ static void plugin_ui_show_load_err(int err_type, gchar *loadfile)
       if (temp) *temp = '\0';
       gchar *msg = g_strdup_printf(_("game_ni_unsupported_file%s"), basename);
 
+      
       gtk_window_set_title(GTK_WINDOW(gs.ui->hildon_appview), "");  
+      plugin_settings_set_string(SETTINGS_CHESS_SAVE_FILE,"");
       plugin_show_infoprint(msg);
       
       g_free(msg);
@@ -1554,6 +1513,19 @@ void game_load(gchar *filename)
         read_handle=NULL;    
     }
 
+    plugin_settings_set_bool(SETTINGS_CHESS_LOAD_IN_PROGRESS, TRUE);
+
+    /* Validate file */
+    if( !validate_chess_file(loadfile) ) {
+        gs.startup_ui_state_change_cb(
+            gs.ui->app, GAME_CLOSED, GAME_CLOSED, gs.ui);
+        plugin_ui_show_load_err(GAME_LOAD_FILE_UNSUPPORTED, loadfile);
+        plugin_settings_set_string(SETTINGS_CHESS_SAVE_FILE,"");
+    	g_free(loadfile);
+    	return;
+    }
+        
+
     gchar *tname=NULL;
     tname=comapp_common_get_display_name(filename,NULL,TRUE);
     if (tname)
@@ -1699,11 +1671,8 @@ static void game_save(void)
     	if (vfs_res == GNOME_VFS_OK ||
         	file_info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
       		
-      		/* MAGICAL RULE OF 2%, SOoOOoo BAD */
       		if (0.02>=((double)info.f_bavail/(double)info.f_blocks)) {
 		      	//no more free space
-        		/* gtk_window_set_title(GTK_WINDOW(gs.ui->hildon_appview), ""); */
-        		//plugin_show_infoprint(__("sfil_ni_not_enough_memory"));
         		g_warning( "Remove this code" );
         		ui_show_nomem_dialog();
         		g_free(unuried_fname);
@@ -1802,7 +1771,7 @@ static void game_save(void)
       case GNOME_VFS_ERROR_ACCESS_DENIED:
       case GNOME_VFS_ERROR_READ_ONLY:
       case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
-		hildon_banner_show_information( gs.ui->hildon_appview,
+		hildon_banner_show_information( GTK_WIDGET(gs.ui->hildon_appview),
 			NULL, dgettext("hildon-fm", "sfil_ib_readonly_location" ));      
         chess_saved = FALSE;      	
       	break;
@@ -1824,25 +1793,21 @@ static void game_save(void)
 
 
 static void plugin_ui_cancel_saving(void) {
-//  AppData *app_data = get_app_data();
-  
-  /*app_data->app_ui_data->*/chess_save_done = GAME_SAVE_CANCEL;
+  chess_save_done = GAME_SAVE_CANCEL;
   
   /* wait chess engine return result after saved */ 	  
-  while (/*app_data->app_ui_data->*/chess_save_done == GAME_SAVE_CANCEL) {
+  while (chess_save_done == GAME_SAVE_CANCEL) {
     while(gtk_events_pending())
       gtk_main_iteration();
     usleep(10000);
   }
-  plugin_ui_hide_saving_dialog(/*app_data->app_ui_data*/);  
+  plugin_ui_hide_saving_dialog();  
 
   /* Was handled by game_save() */
-  if (/*app_data->app_ui_data->*/chess_save_done==GAME_SAVE_NONE) {
+  if (chess_save_done==GAME_SAVE_NONE) {
     return;
   }
   
-  //ui_repaint(app_data);
-  
   /* Delete un-wanted file if file is new */
   if (is_new_file) {
     gchar *filename = settings_get_string(SETTINGS_CHESS_SAVE_FILE);
@@ -1921,11 +1886,6 @@ void ui_show_nomem_dialog(void)
   
   gtk_dialog_run(GTK_DIALOG(note));
   gtk_widget_destroy(GTK_WIDGET(note));
-
-  /* Dialog shouldn't do this
-  gtk_window_set_title(GTK_WINDOW(gs.ui->hildon_appview), "");
-  plugin_settings_set_string(SETTINGS_CHESS_SAVE_FILE,"");
-  */
 }
 
 
@@ -1940,7 +1900,6 @@ add_recent_item(gchar *file)
 /* Get string type key value */
 gchar* settings_get_string(const gchar *key)
 {
-  //osso_log(LOG_DEBUG,"Getting key %s value as string\n", key);
   return gconf_client_get_string(gcc, key, NULL);
 }
 
@@ -2040,30 +1999,27 @@ void game_load_recent_cb(ComappRecent *comapprecent,gchar* file_uri,gpointer use
 }
 
 static void
-file_monitor_cb( GnomeVFSMonitorHandle * handle, const gchar * monitor_uri,
-	const gchar * info_uri, GnomeVFSMonitorEventType event_type,
-	gpointer user_data ) 
+file_monitor_cb( GnomeVFSMonitorHandle *handle, const gchar * monitor_uri,
+	const gchar *info_uri, GnomeVFSMonitorEventType event_type,
+	gpointer user_data) 
 {
 
-    osso_log(LOG_DEBUG,"file_monitor_cb: enter");    
 	/* If file is changed to read only */
     gboolean sensitivity;
 	if( event_type == GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED ) {
 		GnomeVFSFileInfo info;
-        osso_log(LOG_DEBUG,"file_monitor_cb: metadata changed");    
 		
 		if( gnome_vfs_get_file_info( monitor_uri, &info,
 			GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS ) == GNOME_VFS_OK ) {
-            osso_log(LOG_DEBUG,"file_monitor_cb: get access rights = vfs ok");    
 		
             if (save_item != NULL) {    
                 sensitivity = ( info.permissions &  GNOME_VFS_PERM_ACCESS_WRITABLE ) ==  GNOME_VFS_PERM_ACCESS_WRITABLE;
-                osso_log(LOG_DEBUG,"file_monitor_cb: saveitem not null, setting sensitivity %d",sensitivity);    
     			gtk_widget_set_sensitive( save_item, sensitivity );
             }
 		}
-		
 	}
-    osso_log(LOG_DEBUG,"file_monitor_cb: leave");    
 
+	handle = 0;
+	info_uri = 0;
+	user_data = 0;
 }
