@@ -85,8 +85,8 @@ gboolean main_win_expose(GtkWidget      *widget,
                                             GdkEventExpose *event,
                                             gpointer        user_data);
 
-gboolean main_win_expose(GtkWidget      *widget,
-                                            GdkEventExpose *event,
+gboolean main_win_expose(GtkWidget      *,
+                                            GdkEventExpose *,
                                             gpointer        user_data)
 {
   AppUIData *app_ui_data=(AppUIData*)user_data;
@@ -269,7 +269,7 @@ GtkWindow *ui_create_main_view( void )
 
 
 gboolean end_dialog_focus_out(GtkWidget *widget,
-                              GdkEventFocus *event,
+                              GdkEventFocus *,
                               AppUIData *app_ui_data)
 {
     if (!focus_is_own())
